""" file:   documentation.py (earthchem)
    author: Jess Robertson, CSIRO Minerals
    date:   May 2018

    description: Scraping the Earthchem site for documentation etc
"""

import requests
from bs4 import BeautifulSoup

from collections import OrderedDict
import pkg_resources
import re
import textwrap

def strip_whitespace(string):
    """ Strip newline, tab and multple whitespace from a string
    """
    return re.sub(' +', ' ',
                  string.replace('\n', ' ').replace('\t', ' ').strip())


# Construct request from EarthChem rest documentation
REST_DOCO_URL = 'http://ecp.iedadata.org/rest_search_documentation/'
if not requests.get(REST_DOCO_URL).ok:
    # We can just use the cached version
    CACHED_DOCO_FILE = pkg_resources.resource_stream(
        "earthchem.resources",
        "earthchem_rest_search_documentation.html")
    REST_DOCO_URL = 'file://' + str(CACHED_DOCO_FILE)

# Keys to ignore when constructing the query class
IGNORE_VALUES = (
    re.compile('Example.*'),
    re.compile('level[0-9]')
)

def get_documentation():
    """ Get query items and documentaton by scraping the EarthChem rest
        documentation
    """
    # Hit the endpoint
    response = requests.get(REST_DOCO_URL)
    if response.ok:
        soup = BeautifulSoup(response.text, 'lxml')
    else:
        raise IOError("Can't find Earthchem REST documentation")

    # Parse me some documentation
    docs = OrderedDict()
    for item in soup.select('.itemtitle'):
        # Check that we actually want to keep this value
        itemname = strip_whitespace(item.contents[0])
        if any(map(lambda regex: regex.match(itemname),
                   IGNORE_VALUES)):
            continue

        # Parse document string, add to dictionary
        itemdoc = strip_whitespace(item.contents[1].contents[0])
        docs[itemname] = itemdoc

    return docs



""" file:   requests.py
    author: Jess Robertson, CSIRO Minerals
    date:   May 2018

    description: Handles requests against EarthChem's API
"""

from .documentation import get_documentation

import requests
import pandas

from io import StringIO
import textwrap

def make_query_docstring():
    """ Constructs a docstring from the documentation dictionary
    """
    wrapper = textwrap.TextWrapper(width=80, subsequent_indent='    ')
    docstr = textwrap.dedent("""
        Holds a query for the EarthChem REST API

        Initialize by providing key-value pairs to build into a query URL. The
        URL is available in the `url` attribute, and the results from the
        `results` attribute.

        Providing a keyword not in the list below will raise a KeyError.

        Allowed keywords are:
        """)
    docdict = get_documentation()
    for item in docdict.items():
        docstr += '\n' + wrapper.fill('{0} - {1}'.format(*item))
    return docstr

class RESTClientQuery(dict):

    __doc__ = make_query_docstring()
    docdict = get_documentation()

    def __init__(self, **kwargs):
        super().__init__()

        # Add everything to dictionary
        for key, value in kwargs.items():
            # Add to dictionary
            self[key] = str(value)

    def __repr__(self):
        kwargs = ', '.join('{0}={1}'.format(*it) for it in self.items())
        return 'RESTClientQuery({})'.format(kwargs)

    def __setitem__(self, key, value):
        """ Sets a particular query term, making sure that the values 
            are ok etc. 
            
            Parameters:
                key - the query key to set
                value - the value to set for that search.
        """
        # Check that items are ok to query
        if key not in self.docdict.keys():
            raise KeyError('Unknown key {0}'.format(key))

        if value is None:
            del self[key]
        else:
            super().__setitem__(key, value)

    def count(self):
        """ Get the total number of items returned by the query
        """
        self['searchtype'] = 'count'
        resp = requests.get(self.url)
        # del self['searchtype']

        # Return the result
        if resp.ok:
            try:
                return int(resp.json()['Count'])
            except:
                raise IOError("Couldn't parse data in response")
        else:
            raise IOError("Couldn't get data from network") 

    def dataframe(self, standarditems=True):
        """ Get the actual data in a dataframe

            Note that this doesn't do pagination yet...
        """
        # Add the proper search type keys to the query
        self['searchtype'] = 'rowdata'
        self['standarditems'] = 'yes' if standarditems else 'no'
        resp = requests.get(self.url)
        self['searchtype'], self['standarditems'] = None, None

        # Return the result
        if resp.ok:
            try:
                return pandas.read_json(StringIO(resp.text))
            except ValueError:
                if resp.text == 'no results found':
                    print("Didn't find any records, returning None")
                    return None
                else:
                    raise IOError("Couldn't parse data in response")
        else:
            raise IOError("Couldn't get data from network")

    @property
    def url(self):
        query_string = ('http://ecp.iedadata.org/restsearchservice?'
                        'outputtype=json')
        for item in self.items():
            query_string += '&{0}={1}'.format(*item)
        return query_string
    
    def info(self, key, pprint=True):
        """ Return info about a search key
        
            Parameters:
                key - the key to get information about
                pprint - whether to print the information or return 
                    a dictionary with the contents
                
            Returns:
                if pprint=True, None, otherwise a dictionary with a
                'doc' string and a 'valid_values' 
        """
        pass



#!/usr/bin/env python3
""" file:   setup.py (earthchem)
    author: Jess Robertson
            CSIRO Mineral Resources
    date:   October 2016

    description: Setuptools installer script for earthchem.
"""

from setuptools import setup, find_packages
import versioneer

with open('README.md', 'r') as src:
    LONG_DESCRIPTION = src.read()

## PACKAGE INFORMATION
setup(
    name='earthchem',
    version=versioneer.get_version(),
    description='Data slurper for getting stuff from Earthchem services',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    license='MIT',
    author='Jess Robertson',
    author_email='jesse.robertson@csiro.au',
    url='https://github.com/jesserobertson/earthchem-pyclient.git',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    project_urls={
        'Test coverage': 'https://coveralls.io/github/jesserobertson/earthchem-pyclient',
        'Test status': 'https://travis-ci.org/jesserobertson/earthchem-pyclient'
    },

    # Dependencies
    install_requires=[
        'geopandas',
        'requests',
        'beautifulsoup4',
        'lxml'
    ],
    extras_require={
        'dev': [
            'versioneer',
            'nbstripout',
            'nbdime'
        ]
    },
    tests_require=[
        'pytest',
        'pytest-runner',
        'pytest-cov',
        'coverage'
    ],

    # Contents
    packages=find_packages(exclude=['test*']),
    test_suite="tests",
    package_data={},

    # other stuff
    cmdclass=versioneer.get_cmdclass(),

    # Some entry points for running rosedb
    entry_points={
        'console_scripts': [
            'echem = cli:cli',
        ],
    }
)
